#include "cbignumber.hpp"

//-----------------------------------------------------------------------------
CBigNumber::CBigNumber(const string& str)
//-----------------------------------------------------------------------------
{
    initializeBigNum(str);
}

//-----------------------------------------------------------------------------
CBigNumber::~CBigNumber()
//-----------------------------------------------------------------------------
{

}

//-----------------------------------------------------------------------------
void CBigNumber::setNumber(const string& str)
//-----------------------------------------------------------------------------
{
    initializeBigNum(str);
}

//-----------------------------------------------------------------------------
const string& CBigNumber::getNumberString() const
//-----------------------------------------------------------------------------
{
    return numberInString_;
}

//-----------------------------------------------------------------------------
const vector<int>& CBigNumber::getNumberVector() const
//-----------------------------------------------------------------------------
{
    return numbersArray_;
}

//-----------------------------------------------------------------------------
CBigNumber CBigNumber::operator +(const CBigNumber &bigNumber) const
//-----------------------------------------------------------------------------
{
    CBigNumber returnedValue;
    const CBigNumber * longerNumber = this;
    const CBigNumber * shorterNumber = &bigNumber;

    if (*this < bigNumber)
    {
        swap(longerNumber, shorterNumber);
    }

    returnedValue.numbersArray_.reserve(longerNumber->getNumberVector().size());
    returnedValue.numbersArray_ = longerNumber->getNumberVector();

    auto firstNumberIterator = longerNumber->numbersArray_.end();
    auto secondNumberIterator = shorterNumber->numbersArray_.end();
    auto returnedValueIterator = returnedValue.numbersArray_.end();

    int counter = shorterNumber->getNumberVector().size() + 1;
    while (0 < counter)
    {
        *returnedValueIterator = *firstNumberIterator + *secondNumberIterator;
        --firstNumberIterator;
        --secondNumberIterator;
        --returnedValueIterator;
        --counter;
    }

    returnedValue.alignNumberCells();
    returnedValue.numberInString_ = returnedValue.convertArrayNumFormToString();

    return returnedValue;
}

//-----------------------------------------------------------------------------
bool CBigNumber::operator <(const CBigNumber &bigNumber) const
//-----------------------------------------------------------------------------
{
    bool returnedValue;
    int sizeOfFirstNumber = numbersArray_.size();
    int sizeOfSecondNumber = bigNumber.getNumberVector().size();

    if (1 == sizeOfFirstNumber && 1 == sizeOfSecondNumber)
    {
        if (bigNumber.getNumberVector()[0] == numbersArray_[0])
        {
            returnedValue = false;
        }
    }
    else
    {
        returnedValue = isFirstNumberLowest(bigNumber, *this);
    }

    return returnedValue;
}

//-----------------------------------------------------------------------------
string CBigNumber::convertArrayNumFormToString()
//-----------------------------------------------------------------------------
{
    string returnedValue;

    for (const int& number : numbersArray_)
    {
        returnedValue += to_string(number);
    }

    return returnedValue;
}

//-----------------------------------------------------------------------------
void CBigNumber::initializeBigNum(const string& str)
//-----------------------------------------------------------------------------
{
    if (isStringValid(str))
    {
        numbersArray_.clear();
        for (const char& latter : str)
        {
            if (isdigit(latter))
            {
                numbersArray_.push_back(latter - '0');
            }
        }
        convertArrayNumFormToString();
    }
}

//-----------------------------------------------------------------------------
void CBigNumber::alignNumberCells()
//-----------------------------------------------------------------------------
{
    numbersArray_.reserve(numbersArray_.capacity() + 1);
    auto numberItterator = numbersArray_.end();
    --numberItterator;
    while (true)
    {
        if (*numberItterator > 9)
        {
            int greateOrder = *numberItterator / 10;
            *numberItterator = *numberItterator % 10;
            if (numbersArray_.begin() != numberItterator)
            {
                auto nextNumber = numberItterator - 1;
                *nextNumber += greateOrder;
            }
            else
            {
                numbersArray_.insert(numbersArray_.begin(), greateOrder);
            }
        }
        if (numbersArray_.begin() == numberItterator)
        {
            break;
        }
        else
        {
            --numberItterator;
        }
    }
}

//-----------------------------------------------------------------------------
bool CBigNumber::isStringValid(const string& str) const
//-----------------------------------------------------------------------------
{
    bool returnValue = false;

    for (char latter : str)
    {
        if (isdigit(latter))
        {
            returnValue = true;
        }
    }

    return returnValue;
}

//-----------------------------------------------------------------------------
bool CBigNumber::isFirstNumberLowest(const CBigNumber &firstNumber, const CBigNumber &secondNumber) const
//-----------------------------------------------------------------------------
{
    bool returnedValue;
    int firstNumberArraySize = firstNumber.getNumberVector().size();
    int secondNumberArraySize = secondNumber.getNumberVector().size();

    returnedValue = true;

    if (firstNumberArraySize < secondNumberArraySize)
    {
        returnedValue = false;
    }

    if (firstNumberArraySize == secondNumberArraySize)
    {
        int firstDigitOfFirstNumber = firstNumber.getNumberVector()[0];
        int secondDigitOfFirstNumber = secondNumber.getNumberVector()[0];

        if (firstDigitOfFirstNumber < secondDigitOfFirstNumber)
        {
            returnedValue = false;
        }
    }

    return returnedValue;
}
